"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateAnimatedValue = exports.validateHyUI = exports.validateVersionList = exports.getVersionFromStr = exports.postAnimatedWarns = exports.minValidVersion = void 0;
var child_process_1 = require("child_process");
var semver_1 = __importDefault(require("semver"));
var axios_1 = __importDefault(require("axios"));
var os_1 = require("os");
exports.minValidVersion = '1.3.29';
var excludeList = [
    '**/node_modules/react-native/*',
    '**/node_modules/react-native-web/*',
    '**/node_modules/@hyext/hy-ui/*',
];
function postAnimatedWarns(files, extUuid) {
    var content = "".concat(extUuid, " \u4F7F\u7528\u4E86 new Animated.Value, \u6587\u4EF6\u5217\u8868: \n-").concat(files.join('\n-'));
    if (!process.env.RN_ANIMATED_URL)
        return;
    try {
        (0, axios_1.default)({
            method: 'post',
            url: process.env.RN_ANIMATED_URL,
            data: {
                msgtype: 'text',
                text: {
                    content: content,
                },
            },
        });
    }
    catch (err) {
        console.warn('告警发送失败');
    }
}
exports.postAnimatedWarns = postAnimatedWarns;
var getVersionFromStr = function (content, pkgName) {
    var _a, _b;
    if (pkgName === void 0) { pkgName = '@hyext/hy-ui'; }
    var reg = new RegExp("".concat(pkgName, "@(.+)$"), 'i');
    var lines = (_b = (_a = content === null || content === void 0 ? void 0 : content.split(os_1.EOL)) === null || _a === void 0 ? void 0 : _a.map(function (xs) { var _a; return (_a = reg.exec(xs)) === null || _a === void 0 ? void 0 : _a[1]; })) === null || _b === void 0 ? void 0 : _b.filter(function (xs) { return !!xs; });
    return lines;
};
exports.getVersionFromStr = getVersionFromStr;
var validateVersionList = function (list, baseVersion) {
    if (baseVersion === void 0) { baseVersion = exports.minValidVersion; }
    return list.every(function (xs) { return semver_1.default.satisfies(xs, ">=".concat(baseVersion), { includePrerelease: true }); });
};
exports.validateVersionList = validateVersionList;
function validateHyUI(projectRoot, baseVersion) {
    if (baseVersion === void 0) { baseVersion = exports.minValidVersion; }
    var pkgName = '@hyext/hy-ui';
    //   ~/Downloads took 4s
    // ❯ npm --prefix ~/Developer/huya/hyext/just-dance ls @hyext/hy-ui --parseable --long
    // /Users/simon/Developer/huya/hyext/just-dance/node_modules/@hyext/hy-ui:@hyext/hy-ui@1.3.26
    var _a = (0, child_process_1.spawnSync)('npm', [
        'ls',
        pkgName,
        '--parseable',
        '--long',
    ], {
        cwd: projectRoot,
    }), stdout = _a.stdout, stderr = _a.stderr, status = _a.status, error = _a.error;
    if (error)
        return true;
    return (0, exports.validateVersionList)((0, exports.getVersionFromStr)(stdout === null || stdout === void 0 ? void 0 : stdout.toString()), baseVersion);
}
exports.validateHyUI = validateHyUI;
function validateAnimatedValue(projectRoot, extUuid, exclude) {
    var _a, _b;
    if (exclude === void 0) { exclude = excludeList; }
    // rg -uuuu -l 'new Animated.Value' ~/Developer/huya/hyext/just-dance --type js -g '!{**/node_modules/react-native/*,**/node_modules/react-native-web/*,**/node_modules/@hyext/hy-ui/*}'
    var _c = (0, child_process_1.spawnSync)('rg', [
        '-uuuuu',
        '--files-with-matches',
        'new Animated.Value',
        projectRoot,
        '--type',
        'js',
        '-g',
        "!{".concat(exclude.join(','), "}"),
    ], {
        cwd: projectRoot,
    }), stdout = _c.stdout, status = _c.status, error = _c.error;
    if (error) {
        console.warn('validateAnimatedValue error', error.message);
    }
    var files = (_b = (_a = stdout === null || stdout === void 0 ? void 0 : stdout.toString()) === null || _a === void 0 ? void 0 : _a.split(os_1.EOL)) === null || _b === void 0 ? void 0 : _b.filter(function (xs) { return !!xs; });
    if (files.length > 0) {
        postAnimatedWarns(files.map(function (xs) { return xs.replace("".concat(projectRoot, "/"), ''); }), extUuid);
    }
}
exports.validateAnimatedValue = validateAnimatedValue;
