// @ts-ignore
import DependencyGraph from 'metro/src/node-haste/DependencyGraph'
// @ts-ignore
import Transformer from 'metro/src/DeltaBundler/Transformer'
// @ts-ignore
import DeltaCalculator from 'metro/src/DeltaBundler/DeltaCalculator'

import path from 'path'

// @ts-ignore
import saveAssets from '@react-native-community/cli-plugin-metro/build/commands/bundle/saveAssets'

import { Platform } from '../lib/generateBuildResult'
import saveOutput from './saveOutput'
import { TransformInputOptions } from 'metro'
import { calcTransformerOptions } from './calcTransformerOptions'
import { getType } from './getType'
import { getAssets } from './getAssets'
import { getPrepended } from './getPrepended'
import { BundleConfigItem, CompileExtOutput, ExtMetroConfig } from '../matrix-config/configTypes'
import { skipModulePlaceHolder } from '../matrix-config/getExtMetroConfig'

// metro/src/shared/types.flow.js
export type SerializerOptions = {
  sourceMapUrl?: string
  sourceUrl?: string
  runModule: boolean
  excludeSource: boolean
  inlineSourceMap: boolean
  modulesOnly: boolean
}

export async function batchCompile({
  config,
  bundleConfigs,
}: { config: ExtMetroConfig, bundleConfigs: BundleConfigItem[] }): Promise<CompileExtOutput> {
  // custom 1300ms/20_000ms
  // !custom 5000ms/138_800ms
  // 1300ms
  //
  // const graphOptions = { shallow: false }

  const serializerOptions: SerializerOptions = {
    excludeSource: false,
    inlineSourceMap: false,
    modulesOnly: false,
    runModule: true,
    sourceMapUrl: undefined,
    sourceUrl: undefined,
  }

  const dependencyGraph = await DependencyGraph.load(config, { watch: false })
  const transformer = new Transformer(config, dependencyGraph.getSha1.bind(dependencyGraph))

  const absoluteEntryFiles: string[] = bundleConfigs.map(xs => xs.entryAbsPath)

  const getPlatformGraph = async (platform: Platform) => {
    const transformOptions: TransformInputOptions = {
      customTransformOptions: Object.create(null),
      dev: false,
      hot: false,
      minify: true,
      platform,
      runtimeBytecodeVersion: undefined,
      type: 'module',
      unstable_transformProfile: 'default',
    }

    const calculatedOptions = await calcTransformerOptions(config, transformOptions)



    const deltaCalculator = new DeltaCalculator(absoluteEntryFiles, dependencyGraph, {
      resolve: (from: string, to: string) => dependencyGraph.resolveDependency(from, to, platform),
      transform: (filepath: string) => {
        if (!config.serializer.processModuleFilter({ path: filepath } as any)) {
          return Promise.resolve(skipModulePlaceHolder)
        }

        return transformer.transformFile(
          filepath,
          {
            ...calculatedOptions,
            type: getType(config, calculatedOptions, filepath),
            inlineRequires: false,
          })
      },
      transformOptions,
      onProgress: null,
      experimentalImportBundleSupport: false,
      shallow: false,
    })
    await deltaCalculator.getDelta({ reset: true, shallow: false })
    const graph = deltaCalculator.getGraph()

    config.serializer?.experimentalSerializerHook?.(graph, {
      added: graph.dependencies,
      modified: new Map(),
      deleted: new Set(),
      reset: true,
    })

    const _getAssets = () => {
      if (platform === 'ios') {
        return getAssets({
          entryFiles: absoluteEntryFiles,
          config,
          transformOptions,
          dependencyGraph,
          transformer,
        })
      }
      return []
    }

    const [prepend, assets] = await Promise.all([
      getPrepended({
        config,
        transformOptions,
        dependencyGraph,
        platform,
        transformer,
        extraVars: {
          __HYEXT_RN_VER__: `${process.env.HYEXT_RN_VER || '0.67.2'}`,
          HYEXT_BUILD_ENV: `${process.env.HYEXT_BUILD_ENV || 'dev'}`,
          HYEXT_BUILD_TYPE: 'rn',
          HYEXT_IMAGE_BASE_URL: config.imageBaseURL!.endsWith('/')
            ? config.imageBaseURL!.slice(0, config.imageBaseURL!.length - 1)
            : config.imageBaseURL!,
        },
      }),
      _getAssets(),
    ])

    return { graph, prepend, deltaCalculator, platform, assets }
  }

  // 这里用 Promise.all 对速度并没有任何提升, 还不如一个个来比较稳妥
  const graphList: Array<Awaited<ReturnType<typeof getPlatformGraph>>> = []
  graphList.push(await getPlatformGraph('ios'))
  graphList.push(await getPlatformGraph('android'))

  const promiseFactories: any = []
  bundleConfigs.forEach(xs => {
    graphList.forEach(grph => {
      promiseFactories.push(() => saveOutput({
        entryAbsPath: xs.entryAbsPath,
        outputAbsPath: path.join(xs.outputDir, `index.${grph.platform}.jsbundle`),
        dependencyGraph,
        graph: grph.graph,
        config,
        serializerOptions,
        prepend: grph.prepend,
        platform: grph.platform,
      }))
    })
  })


  // @ts-ignore
  const outputList = await promiseFactories.reduce((queue, cur) => {
    // @ts-ignore
    return queue.then(acc => cur().then(result => ([...acc, result])))
  }, Promise.resolve([]))

  const assets = graphList.find(xs => xs.platform === 'ios')?.assets
  await saveAssets(assets, 'ios', config.assetsDir)

  graphList.forEach(({ deltaCalculator }) => deltaCalculator.end())
  transformer.end()
  dependencyGraph.end()

  return outputList
}
