export function bundleToString(bundle: {
  post: string
  pre: string
  modules: Array<[number | string, string]>
}): {
  code: string,
  metadata: { pre: number, post: number, modules: Array<[number | string, number]> }
} {
  let code = bundle.pre.length > 0 ? bundle.pre + '\n' : ''
  const modules: Array<[number | string, number]> = []

  const sortedModules = bundle.modules
    .slice()
    .sort((a: [number | string, string], b: [number | string, string]) => {
      if (a > b) return 1
      if (a < b) return -1

      return 0
    })

  for (const [id, moduleCode] of sortedModules) {
    if (moduleCode.length > 0) {
      code += moduleCode + '\n'
    }
    modules.push([id, moduleCode.length])
  }

  if (bundle.post.length > 0) {
    code += bundle.post
  } else {
    code = code.slice(0, -1)
  }

  return {
    code,
    metadata: { pre: bundle.pre.length, post: bundle.post.length, modules },
  }
}
