import { TransformInputOptions } from "metro"
import { ConfigT } from "metro-config"
// @ts-ignore
import DependencyGraph from 'metro/src/node-haste/DependencyGraph'
// @ts-ignore
import Transformer from 'metro/src/DeltaBundler/Transformer'
// @ts-ignore
import DeltaCalculator from 'metro/src/DeltaBundler/DeltaCalculator'

import { calcTransformerOptions } from "./calcTransformerOptions"
import { getType } from "./getType"
import { skipModulePlaceHolder } from "../matrix-config/getExtMetroConfig"

export async function getDependencies(options: {
  dependencyGraph: DependencyGraph,
  config: ConfigT,
  entryFiles: string[],
  transformOptions: TransformInputOptions,
  transformer: Transformer,
  otherOptions?: {
    onProgress: any,
    shallow: boolean,
  }
}
) {
  const {
    dependencyGraph,
    config,
    entryFiles,
    transformOptions,
    transformer,
    otherOptions = {
      onProgress: null,
      shallow: false,
    }
  } = options

  const calculatedOptions = await calcTransformerOptions(
    config,
    transformOptions, // transformOptions.type: 'script' | 'module' | 'asset';
  )

  const deltaCalculator = new DeltaCalculator(
    entryFiles,
    dependencyGraph,
    {
      resolve: (from: string, to: string) => dependencyGraph.resolveDependency(from, to, transformOptions.platform),
      transform: (filepath: string) => {
        if (!config.serializer.processModuleFilter({ path: filepath } as any)) {
          return Promise.resolve(skipModulePlaceHolder)
        }

        return transformer.transformFile(
          filepath,
          {
            ...calculatedOptions,
            type: getType(config, calculatedOptions, filepath),
            inlineRequires: false,
          })
      },
      transformOptions,
      onProgress: otherOptions.onProgress,
      experimentalImportBundleSupport:
        config.transformer.experimentalImportBundleSupport,
      shallow: otherOptions.shallow,
    })

  await deltaCalculator.getDelta({ reset: true, shallow: otherOptions.shallow })
  const graph = deltaCalculator.getGraph()

  deltaCalculator.end()
  return graph.dependencies
}

