import { TransformInputOptions } from "metro"
import { ConfigT } from "metro-config"
// @ts-ignore
import DependencyGraph from 'metro/src/node-haste/DependencyGraph'
// @ts-ignore
import Transformer from 'metro/src/DeltaBundler/Transformer'
// @ts-ignore
import defaults from 'metro-config/src/defaults/defaults'
// @ts-ignore
import getPreludeCode from 'metro/src/lib/getPreludeCode';
// @ts-ignore
import countLines from 'metro/src/lib/countLines';
// @ts-ignore
// import { compile as hermesCompiler } from 'metro-hermes-compiler'

import { Platform } from "../lib/generateBuildResult"
import { getDependencies } from "./getDependencies"

export const getPrepended = async (options: {
  config: ConfigT,
  transformOptions: TransformInputOptions,
  dependencyGraph: DependencyGraph,
  platform: Platform,
  transformer: Transformer,
  extraVars: { [key: string]: string }
}) => {
  const {
    config,
    transformOptions,
    dependencyGraph,
    platform,
    transformer,
    extraVars,
  } = options

  const polyfillModuleNames = config.serializer
    .getPolyfills({ platform })
    .concat(config.serializer.polyfillModuleNames)

  const dependencies = await getDependencies({
    entryFiles: [defaults.moduleSystem, ...polyfillModuleNames],
    config,
    dependencyGraph,
    transformer,
    transformOptions: {
      ...transformOptions,
      type: 'script',
    }
  })

  const getPrelude = () => {
    const code = getPreludeCode({
      isDev: transformOptions.dev,
      globalPrefix: config.transformer.globalPrefix,
      extraVars,
    })
    const name = '__prelude__'

    return {
      dependencies: new Map(),
      getSource: (): Buffer => Buffer.from(code),
      inverseDependencies: new Set(),
      path: name,
      output: [
        {
          type: 'js/script/virtual',
          data: {
            code,
            lineCount: countLines(code),
            map: [],
          },
        },
        {
          type: 'bytecode/script/virtual',
          data: {
            // TODO: hermesCompiler 有副作用, 引入后会影响当前进程.
            // 1. 进程抛出的错误会被它捕获, 然后仅抛出有限的错误信息
            // 2. 会产生一些奇怪的行为, 比如 babel-loader 中的 AcornParser.extend 会抛出错误:
            // RuntimeError: abort(TypeError: AcornParser.extend is not a function). Build with -s ASSERTIONS=1 for more info
            // bytecode: hermesCompiler(code, { sourceURL: '__prelude__.virtual.js' }).bytecode,
            bytecode: Buffer.from('')
          },
        },
      ],
    }
  }

  return [
    getPrelude(),
    ...dependencies.values(),
  ]
}
