// @ts-ignore
import { mergeConfig } from 'metro-config'

import { dbg } from '../lib/logger'
import path from 'path'
import axios from 'axios'
import { batchCompile } from './batchCompile'
import { CompileExtInput, CompileExtOutput, ExtMetroConfig } from '../matrix-config/configTypes'
import { getExtMetroConfig } from '../matrix-config/getExtMetroConfig'


export default async function compile({
  projectRoot,
  bundleConfigs,
  projectConfig,
  extMetroConfig = {},
}: CompileExtInput): Promise<CompileExtOutput> {
  dbg('compile start')
  const allStart = Date.now()

  const config = getExtMetroConfig({
    projectRoot,
    mode: 'production',
    projectConfig,
    extMetroConfig,
  })

  dbg('config: %O', { ...config, reporter: undefined, filterModules: undefined, filterModulesVersion: undefined })

  process.env.MATRIX_BABEL_PLUGINS = JSON.stringify(config?.babelPlugins || [])
  process.env.MATRIX_HYEXT_TRANSFORMER_CONFIG = JSON.stringify(config?.hyextTransformerConfig || {})

  const result = await batchCompile({
    config,
    bundleConfigs,
  })

  const cost = Date.now() - allStart
  dbg('totalCompileTime: %d', cost)

  if (process.env.MATRIX_BUILD_ID) {
    const axiosConfig = {
      method: 'PUT',
      url: 'http://localhost:3002/telemetry/statExtra',
      timeout: 3000,
      data: {
        buildId: process.env.MATRIX_BUILD_ID,
        extra: [[bundleConfigs.map(xs => path.basename(xs.outputDir)).join(','), cost]]
      }
    }
    dbg('axios Config: %s', JSON.stringify(axiosConfig))
    axios(axiosConfig as any).catch(err => console.error(`ignore telemetry error: ${err.message}`))
  }

  return result
}
