// @ts-ignore
import DependencyGraph from 'metro/src/node-haste/DependencyGraph'
// @ts-ignore
import baseJSBundle from 'metro/src/DeltaBundler/Serializers/baseJSBundle';
// @ts-ignore
import sourceMapString from 'metro/src/DeltaBundler/Serializers/sourceMapString'

import { SerializerOptions } from './batchCompile';
import { ConfigT } from 'metro-config';
import { Platform } from '../lib/generateBuildResult';
import path from 'path'
import { bundleToString } from './bunldeToString';
import { getSortedModules } from './getSortedModules';
import fs from 'fs-extra'
import md5 from '../lib/md5';
import { dbg } from '../lib/logger';

const saveOutput = async (outputOptions: {
  entryAbsPath: string
  outputAbsPath: string
  dependencyGraph: DependencyGraph
  graph: any
  config: ConfigT
  serializerOptions: SerializerOptions
  prepend: any
  platform: Platform
}) => {
  const {
    entryAbsPath,
    outputAbsPath,
    dependencyGraph,
    graph,
    config,
    serializerOptions,
    prepend,
    platform,
  } = outputOptions

  const bundleOptions = {
    asyncRequireModulePath: dependencyGraph.resolveDependency(`${config.projectRoot}/.`, config.transformer.asyncRequireModulePath),
    processModuleFilter: config.serializer.processModuleFilter,
    createModuleId: config.serializer.createModuleIdFactory(),
    getRunModuleStatement: config.serializer.getRunModuleStatement,
    dev: false,
    projectRoot: config.projectRoot,
    modulesOnly: serializerOptions.modulesOnly,
    runBeforeMainModule:
      config.serializer.getModulesRunBeforeMainModule(
        path.relative(config.projectRoot, entryAbsPath),
      ),
    runModule: serializerOptions.runModule,
    sourceMapUrl: serializerOptions.sourceMapUrl || `${path.basename(outputAbsPath)}.map`,
    sourceUrl: serializerOptions.sourceUrl,
    inlineSourceMap: serializerOptions.inlineSourceMap,
    serverRoot: config.projectRoot,
  }

  const bundleCode = bundleToString(
    baseJSBundle(entryAbsPath, prepend, graph, bundleOptions),
  ).code

  const bundleMap = sourceMapString(
    [...prepend, ...getSortedModules(graph, config.serializer.createModuleIdFactory())],
    {
      excludeSource: serializerOptions.excludeSource,
      processModuleFilter: config.serializer.processModuleFilter,
    },
  )

  await fs.ensureDir(path.dirname(outputAbsPath))
  await Promise.all([
    fs.writeFile(outputAbsPath, bundleCode, { encoding: 'utf8' }),
    fs.writeFile(`${outputAbsPath}.map`, bundleMap),
  ])

  return {
    entryAbsPath,
    bundleAbsPath: outputAbsPath,
    bundleMd5: await md5(outputAbsPath),
    platform,
  }
}

export default saveOutput
