// @ts-ignore untyped metro
// import Metro from '@hyext/matrix';
import Metro from 'metro';
// @ts-ignore
import { Terminal } from 'metro-core';
// @ts-ignore
import { mergeConfig } from 'metro-config'

import path from 'path';
// @ts-ignore
import { createDevServerMiddleware, indexPageMiddleware, } from '@react-native-community/cli-server-api';

import enableWatchMode from './watchMode';
import { DEFAULT_HOST } from '../constants'
import HyTerminalReporter from '../lib/HyTerminalReporter'
import { ServerInput } from '../matrix-config/configTypes'
import { hyExtMiddleware } from './hyExtMiddleware'
import { getExtMetroConfig } from '../matrix-config/getExtMetroConfig';

export default async function matrixServer(config: ServerInput) {
  let reportEvent: ((event: any) => void) | undefined;
  const terminal = new Terminal(process.stdout);
  const ReporterImpl = getReporterImpl(config.customLogReporterPath);
  const terminalReporter = new ReporterImpl(terminal);
  const reporter = {
    update(event: any) {
      terminalReporter.update(event);
      if (reportEvent) {
        reportEvent(event);
      }
    },
  };

  const defaultConfig = getExtMetroConfig({
    projectRoot: config.projectRoot,
    mode: 'dev',
    projectConfig: config.projectConfig,
    extMetroConfig: config.extMetroConfig,
  })

  const finalConfig = mergeConfig(
    defaultConfig,
    {
      reporter,
      server: {
        port: config.port || 8081,
      }
    })


  const {
    middleware,
    websocketEndpoints,
    messageSocketEndpoint,
    eventsSocketEndpoint,
  } = createDevServerMiddleware({
    host: config.host || DEFAULT_HOST,
    port: finalConfig.server.port,
    watchFolders: finalConfig.watchFolders,
  });
  middleware.use(indexPageMiddleware);

  const customEnhanceMiddleware = finalConfig.server.enhanceMiddleware;
  finalConfig.server.enhanceMiddleware = (
    metroMiddleware: any,
    server: unknown,
  ) => {
    if (customEnhanceMiddleware) {
      metroMiddleware = customEnhanceMiddleware(metroMiddleware, server);
    }

    // 请保持 middleware 在 metroMiddleware 之前执行
    // 否则 metro 处理 /symbolicate 请求时会得到空的body
    // https://github.com/facebook/metro/issues/660
    return hyExtMiddleware.use(middleware).use(metroMiddleware)
  };

  const serverInstance = await Metro.runServer(finalConfig, {
    host: config.host || DEFAULT_HOST,
    onReady: config.onReady,
    onError: config.onError,
    // @ts-ignore @types/metro 太旧了
    hmrEnabled: true,
    // @ts-ignore @types/metro 太旧了
    websocketEndpoints,
  });

  reportEvent = eventsSocketEndpoint.reportEvent;

  if (config.interactive !== false) {
    enableWatchMode(messageSocketEndpoint);
  }

  serverInstance.keepAliveTimeout = 30000;

  return serverInstance
}

function getReporterImpl(customLogReporterPath: string | undefined) {
  if (customLogReporterPath === undefined) {
    return HyTerminalReporter
  }

  try {
    return require(customLogReporterPath);
  } catch (e) {
    if ((e as any).code !== 'MODULE_NOT_FOUND') {
      throw e;
    }
    return require(path.resolve(customLogReporterPath));
  }
}
